<?php

/* html_functions.php: Some functions for html generation 
*
    Copyright (C) 2002-2005  Hastymail Development group

    This file is part of Hastymail.

    Hastymail is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Hastymail is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hastymail; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

* $Id: html_functions.php,v 1.351 2006/02/16 21:52:46 sailfrog Exp $
*/

/**
 *  function to output folder option elements for select boxes
 *
 *  @param      $mailbox        string|array    currently selected mailbox(s)
 *  @param      $unseen         array|boolean   unseen data array, or false
 *  @param      $simple         boolean         simple display mode flag
 *  @param      $enc            boolean         if true option values are urlencoded
 *  @param      $css_style      string          css style to use for the option elements
 *  @param      $selectable     boolean         true excludes unselectable folders
 *  @param      $to_be_locked   boolean         true makes special folders unselecteable
 *  @param      $raw_output     boolean         if true output only values not option tags
 *  @return                     array           array of formatted html option strings
 *
 */ 
function hm_folder_options2($mailbox, $unseen=false, $simple=false, $enc=false, 
                           $css_style='', $selectable=false, $to_be_locked=false,
                           $raw_output=false) {
                           
    $list = array();
    $delim = $_SESSION['delimiter'];
    $folders = $_SESSION['folders'];
    $dsp_folders = $_SESSION['dsp_folders'];
    if ($to_be_locked && isset($_SESSION['config']['lock_special_folders']) &&
        $_SESSION['config']['lock_special_folders'] == 'true') {
        $locked = true;
    }
    else {
        $locked = false;
    } 
    if ($selectable) {
        $no_sel = array_diff($folders, $selectable);
    }
    elseif (is_array($selectable)) {
        $no_sel = $folders;
    }
    else {
        $no_sel = $_SESSION['nosel_folders'];
    }
    $new_mail_folders = array();
    if (isset($_SESSION['settings']['new_mail_folders'])) {
        $new_mail_folders = $_SESSION['settings']['new_mail_folders'];
    }
    $detail = 'new';
    if (isset($_SESSION['settings']['folder_detail'])) {
        $detail = $_SESSION['settings']['folder_detail'];
    }
    $style = 'nested';
    if (isset($_SESSION['settings']['folder_style'])) {
        $style = $_SESSION['settings']['folder_style'];
    } 
    $names = array();
    if (isset($_SESSION['settings']['namespace'])) {
        $names = $_SESSION['settings']['namespace'];
    }
    foreach($folders as $folder) {
        $out = '';
        if (!$raw_output) {
            $out .= '<option ';
            if (isset($_SESSION['special_folders'])) {
                if (in_array($folder, $_SESSION['special_folders'])) {
                    if ($locked) {
                        $out .= 'disabled ';
                    }
                    $out .= 'style="{font-style: italic}" ';
                }
            }
            $out .= $css_style.' value="';
            if ($enc) {
                $out .= urlencode($folder);
            }
            else {
                $out .= $folder;
            }
            if (in_array($folder, $no_sel)) {
                $out .= ' " DISABLED >';
            }
            elseif (!is_array($mailbox) && 
                    (urlencode($folder) == $mailbox || $folder == $mailbox)) {
                $out .= '" SELECTED >';
            }
            elseif (is_array($mailbox) && in_array($folder, $mailbox)) {
                $out .= '" SELECTED >';
            }
            else {
                $out .= '">';
            }
        }
        if (!$simple && @substr_count($dsp_folders[$folder], $delim) > 0 &&
            $style == 'nested' && !in_array($folder, $names)) {
            $pos = strrpos($dsp_folders[$folder], $delim);
            $out .= str_repeat('&nbsp;', intval($pos/2)).'|_ ';
            if (isset($_SESSION['config']['utf7_decode'])) {
                $out .= utf8_encode(imap_utf7_decode(substr($dsp_folders[$folder],
                     ($pos + 1))));
            }
            else {
                $out .= substr($dsp_folders[$folder], ($pos + 1));
            }
        }
        else {
            if (isset($_SESSION['config']['utf7_decode'])) {
                $out .= utf8_encode(imap_utf7_decode($dsp_folders[$folder]));
            }
            else {
                $out .= $dsp_folders[$folder];
            }
        }
        if ($unseen && $detail != 'none') {
            if (!isset($unseen[$folder])) {
                $unseen[$folder] = 0;
            }
            if (in_array($folder, $new_mail_folders)) {
                if ($detail == 'total') {
                    if (isset($_SESSION['message_count_select'][$folder])) {
                        $out .= '&nbsp;&nbsp;('.$unseen[$folder].
                             '/'.$_SESSION['message_count_select'][$folder].')';
                    }
                    else {
                        if (!isset($_SESSION['message_count'][$folder])) {
                            $out .= '&nbsp;&nbsp;(/)';
                        }
                        else {
                            $out .= '&nbsp;&nbsp;('.$unseen[$folder].
                                    '/'.$_SESSION['message_count'][$folder].')';
                        }
                    } 
                }
                else {
                    $out .= '&nbsp;&nbsp;('.$unseen[$folder].')';
                }
            }
        }
        if (!$raw_output) {
            $out .= '</option>';
        }
        if (!$raw_output) {
            $list[] = $out;
        }
        else {
            $list[urlencode($folder)] = $out;
        }
    }
    return $list;
}

/**
 *  output HTTP headers for a page 
 *  @return                     void
 *
 */
function hm_output_http_headers() {
    header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
    header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT"); 
    header("Cache-Control: no-store, no-cache, must-revalidate");
    header("Cache-Control: post-check=0, pre-check=0", false);
    header("Pragma: no-cache");             
    header("Content-Type: text/html; charset=".$_SESSION['str']['charset']);
}

/**
 *  function to output the top of each page, html title etc. and the toolbar
 *
 *  @param      $imap_stream    resource    conneciton to imap
 *  @param      $mailbox        string      currently selected folder
 *  @param      $unseen         array       unseen array
 *  @param      $refresh        boolean     add refresh meta tag
 *  @param      $page_name      string      name of the calling script
 *  @param      $warning        string      a warning/notice for hm_print_error()
 *  @param      $uid            boolean     if true store current uid in session
 *  @param      $big_form       boolean     true if we don't want to close the HTML form
 *  @return                     array       multi array of header and toolbar values            
 *                  
 */
function hm_toolbar2($imap_stream, $mailbox, $unseen, $refresh=false,
                             $page_name=false, $warning=false, $uid=false,
                             $big_form=false) {
    $toolbar_data = array();
    $head_data = array();
    $config = $_SESSION['config'];
    $settings = $_SESSION['settings'];
    $str = $_SESSION['str'];
    hm_manage_cache_size($str, $page_name, $mailbox);
    if ((isset($_SESSION['settings']['simple']) && $_SESSION['settings']['simple'] == 
        'true') || (isset($_SESSION['auto_simplify']) && $_SESSION['auto_simplify'] == 
        'true')) {
        $simple = true;
    }
    else {
        $simple = false;
    }
    if (isset($settings['refresh_new'])) {
        $refresh_new = $settings['refresh_new'] * 60;
    }
    else { 
        $refresh_new = 60;
    }
    if (isset($config['page_title'])) {
        $title = $config['page_title'];
    }
    else {
        $title = 'Hastymail';
    }
    if ($page_name) {
        $title = $page_name.' | '.$title;
    }
    if ($uid) {
        if (!isset($_SESSION['current_uid'])) {
            $_SESSION['current_uid'] = array();
        }
        $_SESSION['current_uid'][$mailbox] = $uid;
    }
    $total = 0;
    foreach ($unseen as $index => $value) {
        $total += $value;
    }
    if (isset($settings['date_string'])) {
        $date_string = $settings['date_string'];
    }
    else {
        $date_string = 'g:i:s a m/d/y';
    }
    
    /* data for HTML head section */
    $head_data['title_prefix']    = $str[10];
    $head_data['total_unread']    = $total;
    $head_data['page_title']      = $title;
    $head_data['charset']         = $str['charset'];
    $head_data['refresh']         = $refresh;
    $head_data['refresh_new']     = $refresh_new;
    if ($refresh_new > 0) {
        $head_data['refresh_meta']    = '<meta HTTP-EQUIV="refresh" content="'.$refresh_new.
                                        ';url='.$config['hostname'].$config['install_path'].
                                        'new.php?mailbox='.urlencode($mailbox).'&amp;id='.
                                        session_id().'">';
    }
    else {
        $head_data['refresh_meta'] = '';
    }

    /* data for toolbar */
    $toolbar_data['date_string']  = date($date_string, hm_strtotime(date('r')));
    $toolbar_data['go_lbl']       = $str[5];
    $toolbar_data['new_lbl']      = $str[10].' ('.$total.')';
    $toolbar_data['session_id']   = session_id();
    $toolbar_data['search_lbl']   = $str[11];
    $toolbar_data['tb_defaults']  = $_SESSION['toolbar_defaults'];
    $toolbar_data['options_lbl']  = $str[12];
    $toolbar_data['folders_lbl']  = $str[13];
    $toolbar_data['contacts_lbl'] = $str[14];
    $toolbar_data['compose_lbl']  = $str[15];
    $toolbar_data['new_sid_lbl']  = 'New SID';
    $toolbar_data['logout_lbl']   = $str[16];
    $toolbar_data['mailbox_val']  = $mailbox;
    $toolbar_data['hostname']     = $config['hostname'];
    $toolbar_data['install_path'] = $config['install_path'];
    if (isset($settings['wide_bar']) && $settings['wide_bar'] == 'true') {
        $toolbar_data['wide_bar'] = true;
    }
    else {
        $toolbar_data['wide_bar'] = false;
    }
    if (isset($settings['show_clock']) && $settings['show_clock'] == 'true') {
        $toolbar_data['show_clock'] = true;
    }
    else {
        $toolbar_data['show_clock'] = false;
    }
    if (isset($config['iframe']) && $config['iframe'] == 'true' && (!$refresh ||
        $refresh_new == 0)) {
        $toolbar_data['iframe'] = true;
    }
    else {
        $toolbar_data['iframe'] = false;
    }
    $toolbar_data['big_form'] = $big_form;
    $toolbar_data['folder_dropdown_data'] = hm_folder_options2($mailbox, $unseen,
                                       $simple, true);
    $toolbar_data['raw_folder_dropdown_data'] = hm_folder_options2($mailbox, $unseen,
                                       $simple, true, '', false, false, true);
    return (array($toolbar_data, $head_data));
}

/**
 *  Print out one row of message details
 *
 *  @param      $headers        array       message header data
 *  @param      $t_all          boolean     toggle all flag
 *  @param      $mailbox        string      current mailbox
 *  @param      $id             string      session id
 *  @param      $uid            int         message id
 *  @param      $sort_info      array       sort information for thread view
 *  @param      $hilite         boolean     if true highlight row
 *  @param      $chk_folder     string      if set add the folder to the checkbox element
 *  @param      $search         boolean     true if we are outputting search results
 *  @param      $thread         boolean     true if we are outputting thread view results
 *  @param      $current        boolean     true if this message is the last one viewed
 *  @param      $ref            array       thread sorting reference array
 *  @param      $row            int         current row for thread sorting
 *  @param      $s_all          boolean     special flag for toggle all on the new page
 *  @return                     array       array of values need to print out one row
 *
 */
function hm_print_row2($headers, $t_all, $mailbox, $id, $uid, $sort_info=array(),
                      $hilite=true, $chk_folder=false, $search=false, $thread=false, 
                      $current=false, $ref=array(), $row=false, $s_all=false) {
    
    $out = array();
    $str = $_SESSION['str'];
    list ($subject, $from, $date, $priority, $content, 
         $flags, $to, $size, $int_date) = $headers;
    if (!$current) {
        if (isset($_SESSION['current_uid'][$mailbox]) &&
            $uid == $_SESSION['current_uid'][$mailbox]) {
            $current=true;
        }
    }
    if ($subject == '') {
        $subject = $str[229];
    }
    $priority = ltrim($priority);
    $from = hm_address_dsp($from, $str[230]);
    $to = hm_address_dsp($to, $str[232]);
    if (isset($_SESSION['settings']['subject_size']) && strlen($subject) >
        $_SESSION['settings']['subject_size'] && $_SESSION['settings']['subject_size'] >
        0) {
        $subject = substr($subject, 0, $_SESSION['settings']['subject_size']).'...';
    }
    if (strlen($subject) > 120 && strpos($subject, ' ') === false) {
        $subject = hm_text_cleanup($subject, false, true, 70);
    }
    $new = !stristr($flags, 'seen') ? true : false;
    $ans = stristr($flags, 'answered') ? true : false;
    $flagged = stristr($flags, 'flagged') ? true : false;
    $deleted = stristr($flags, 'deleted') ? true : false;
    if ($hilite && $new) {
        $out['class'] = 'tables6';
        $out['font']  = 'font2';
        $out['new'] = true;
    }
    else {
        $out['class'] = 'tables3';
        $out['font']  = 'font1';
        $out['new'] = false;
    }
    $out['searching'] = (!$search && isset($_SESSION['search_info']) &&
                         isset($_SESSION['search_info'][$mailbox][$uid]));
    
    $out['uid'] = $uid;
    $out['deleted'] = $deleted;
    $out['checked'] = false; 
    $out['indent_string'] = false;
    $out['sent_format'] =false;

    /* checkbox */
    if (!$chk_folder) {
        if ($t_all && !isset($_POST[$uid])) $out['checked'] = 1;
    }
    else {
        if ($s_all && isset($_POST[$chk_folder]) && in_array($uid, $_POST[$chk_folder])) {
            $out['checked'] = 1;
        } 
        elseif ($t_all) {
            if (!isset($_POST[$chk_folder]) || !in_array($uid, $_POST[$chk_folder])) {
                $out['checked'] = 1;
            }
        }
    }
    
    $tag = 'mailbox='.urlencode($mailbox).'&amp;id='.urlencode($id).'&amp;new='.
           $new.'&amp;uid='.$uid;
    if ($search) {
        $tag .= '&amp;search=1';
    }
    if ($thread) {
        $tag .= '&amp;thread=1';
    }
    
    $out['tag'] = $tag;
    $out['current'] = $current;
    
    /* thread lines */
    if (isset($sort_info[3]) && isset($sort_info[3][$uid])) {
        if ($sort_info[3][$uid] > 1) {
            $range = array_slice($ref, $row); 
            $remaining = $range;
            $place = array();
            foreach ($range as $index => $val) {
                if ($val < $sort_info[3][$uid]) {
                    $add = true;
                    foreach ($remaining as $v) {
                        if ($v == $val) {
                            $add = true;
                            break;
                        }
                        if ($v < $val) {
                            $add = false;
                            break;
                        }
                    }
                    if ($add) {
                        $place[] = $val;
                    }
                }
            }
            $string = str_repeat("_", $sort_info[3][$uid]);
            foreach ($place as $value) {
                $string{$value} = '|';
            }
            $string = str_replace('_', '&nbsp;&nbsp;&nbsp;', $string);
            $string = str_replace('|', '|&nbsp;&nbsp;&nbsp;', $string);
            $out['indent_string'] = $string;
        }
    }
    /* start of the development mailing list enhancement code
    $subject = preg_replace('/(\[[^]]*\])/i', "<font class=\"font4\">\\1</font>",
               $subject);
    $subject = str_replace(array('Re:', 'RE:', 're:'), '<i>Re:</i>', $subject);
    #$subject = preg_replace('/(RE:)/i', "<i>\\1</i>", $subject);*/

    $out['subject'] = $subject;
    $out['to'] = $to;
    $out['from'] = $from;
    $out['date'] = hm_htmlize($date);
    $out['size'] = hm_size_format($size);
        
    if ((isset($_SESSION['settings']['sent_folder']) &&
       $_SESSION['settings']['sent_folder'] == urldecode($mailbox)) ||
       (isset($_SESSION['settings']['drafts_folder']) &&
       $_SESSION['settings']['drafts_folder'] == urldecode($mailbox) &&
       $mailbox != 'INBOX')) {
       $out['sent_format'] = 1;
    }
    
    $icons = array();
    if (stristr($content, 'multipart') || stristr($content, 'mixed')) $icons[] = '+';
    if ($ans) $icons[] = 'r';
    if ($flagged) $icons[] = 'F';
    if (isset($headers[3]) && $headers[3] < 3) {
        $icons[] = '!';
    }
    $out['icons'] = $icons;
    
    if (strlen($priority) > 0 && $priority{0} < 3) $out['important'] = 1;

    return $out;
}

/**
 *  function to print out the message controls
 *
 *  @param      $dropdown_list  array           message selection options
 *  @param      $number         int             which set of controls (per page)
 *  @param      $dest           string          saved destination folder
 *  @param      $simple         boolean         simple mode flag
 *  @return                     string          data for the message control buttons etc
 *
 */
function hm_print_message_controls2($dropdown_list, $dest, $simple) {
    $str = $_SESSION['str'];
    $message_control_data = array();
    $message_control_data['read_lbl'] = $str[2];
    $message_control_data['unread_lbl'] = $str[3];
    $message_control_data['expunge_lbl'] = $str[363];
    $message_control_data['flag_lbl'] = $str[325];
    $message_control_data['unflag_lbl'] = $str[326];
    $message_control_data['delete_lbl'] = $str[4];
    $message_control_data['copy_lbl'] = $str[1];
    $message_control_data['move_lbl'] = $str[0];
    $message_control_data['skip_trash_lbl'] = $str[311];
    $message_control_data['dropdown'] = $dropdown_list;
    $message_control_data['dest_name'] = 'destination';
    $message_control_data['folders'] = implode('', hm_folder_options2($dest,
                                       false, $simple, false, 'class="option3"'));
    if (isset($_SESSION['settings']['trash_folder']) && 
        isset($_SESSION['settings']['skip_trash']) && !isset($_SESSION['palm'])) {
        $message_control_data['skip_trash_flag'] = true;
    }
    else {
        $message_control_data['skip_trash_flag'] = false;
    } 
    return $message_control_data;
}
?>
